-- -----------------------------------------------------
-- Calender extensions to the Contacts-Addon for OpenEstate-ImmoTool
-- update 3 for HSQLDB
-- Copyright (C) 2009-2019 OpenEstate.org
-- -----------------------------------------------------
-- Rebuild 'view_immotool_contacts_entries_calendar_entries'
-- Rebuild 'remove_immotool_contacts_entries_calendar_entries'
-- Rebuild 'save_immotool_contacts_entries_calendar_entries'
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Rebuild 'view_immotool_contacts_entries_calendar_entries'
-- -----------------------------------------------------
DROP VIEW IF EXISTS view_immotool_contacts_entries_calendar_entries CASCADE;

CREATE VIEW view_immotool_contacts_entries_calendar_entries AS
  SELECT * FROM immotool_contacts_entries_calendar_entries
  WHERE contacts_entry_id IN (SELECT entry_id FROM view_immotool_contacts_entries)
  AND calendar_entry_id IN (SELECT entry_id FROM view_immotool_calendar_entries);

GRANT SELECT
  ON view_immotool_contacts_entries_calendar_entries
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- Rebuild 'remove_immotool_contacts_entries_calendar_entries'
-- -----------------------------------------------------
DROP PROCEDURE IF EXISTS remove_immotool_contacts_entries_calendar_entries CASCADE;
\.
CREATE PROCEDURE remove_immotool_contacts_entries_calendar_entries(
    val_contacts_entry_id BIGINT,
    val_calendar_entry_id BIGINT
  )
  SPECIFIC remove_immotool_contacts_entries_calendar_entries
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE permissions INT;
    DECLARE owner_uid BIGINT;
    DECLARE owner_gid BIGINT;

    IF val_contacts_entry_id < 1 THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    IF val_calendar_entry_id < 1 THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- sicherstellen, dass der Benutzer Lese-Rechte besitzt
    IF NOT is_admin() = TRUE THEN

      -- Lese-Rechte auf der Adresse müssen vorliegen
      SELECT access_permissions, access_owner_id, access_group_id
        INTO permissions, owner_uid, owner_gid
        FROM immotool_contacts_groups
        WHERE group_id IN (SELECT group_id FROM immotool_contacts_entries WHERE entry_id = val_contacts_entry_id);
      IF NOT can_read( permissions, owner_uid, owner_gid ) = TRUE THEN
        SIGNAL SQLSTATE '45000';
      END IF;

      -- Lese-Rechte auf dem Kalendereintrag müssen vorliegen
      SELECT access_permissions, access_owner_id, access_group_id
        INTO permissions, owner_uid, owner_gid
        FROM immotool_calendar_groups
        WHERE group_id IN (SELECT group_id FROM immotool_calendar_entries WHERE entry_id = val_calendar_entry_id);
      IF NOT can_read( permissions, owner_uid, owner_gid ) = TRUE THEN
        SIGNAL SQLSTATE '45000';
      END IF;
    END IF;

    -- Relation entfernen
    DELETE
      FROM immotool_contacts_entries_calendar_entries
      WHERE contacts_entry_id = val_contacts_entry_id
      AND calendar_entry_id = val_calendar_entry_id;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE remove_immotool_contacts_entries_calendar_entries
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- Rebuild 'save_immotool_contacts_entries_calendar_entries'
-- -----------------------------------------------------
DROP PROCEDURE IF EXISTS save_immotool_contacts_entries_calendar_entries CASCADE;
\.
CREATE PROCEDURE save_immotool_contacts_entries_calendar_entries(
    val_contacts_entry_id BIGINT,
    val_calendar_entry_id BIGINT,
    val_relation_type VARCHAR(100),
    val_relation_notes LONGVARCHAR
  )
  SPECIFIC save_immotool_contacts_entries_calendar_entries
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Relation ggf. entfernen
    CALL remove_immotool_contacts_entries_calendar_entries( val_contacts_entry_id, val_calendar_entry_id );

    -- Relation einfügen
    INSERT INTO immotool_contacts_entries_calendar_entries (
      contacts_entry_id,
      calendar_entry_id,
      relation_type,
      relation_notes )
    VALUES (
      val_contacts_entry_id,
      val_calendar_entry_id,
      val_relation_type,
      val_relation_notes );

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_contacts_entries_calendar_entries
  TO "IMMOTOOL";
